@@ -74,8 +74,6 @@ import sys
 
 from mod_pywebsocket import common
 from mod_pywebsocket.extensions import DeflateFrameExtensionProcessor
-from mod_pywebsocket.handshake._base import format_extensions
-from mod_pywebsocket.handshake._base import parse_extensions
 from mod_pywebsocket.stream import Stream
 from mod_pywebsocket.stream import StreamHixie75
 from mod_pywebsocket.stream import StreamOptions
@@ -357,7 +355,7 @@ class ClientHandshakeProcessor(ClientHandshakeBase):
             fields.append(
                 '%s: %s\r\n' %
                 (common.SEC_WEBSOCKET_EXTENSIONS_HEADER,
-                 format_extensions(extensions_to_request)))
+                 common.format_extensions(extensions_to_request)))
 
         for field in fields:
             self._socket.sendall(field)
@@ -452,7 +450,8 @@ class ClientHandshakeProcessor(ClientHandshakeBase):
             common.SEC_WEBSOCKET_EXTENSIONS_HEADER.lower())
         accepted_extensions = []
         if extensions_header is not None and len(extensions_header) != 0:
-            accepted_extensions = parse_extensions(extensions_header[0])
+            accepted_extensions = common.parse_extensions(extensions_header[0])
+        # TODO(bashi): Support the new style perframe compression extension.
         for extension in accepted_extensions:
             extension_name = extension.name()
             if (extension_name == common.DEFLATE_STREAM_EXTENSION and
@@ -493,7 +492,7 @@ class ClientHandshakeProcessorHybi00(ClientHandshakeBase):
     """
 
     def __init__(self, socket, options):
-        super(ClientHandshakeProcessor, self).__init__()
+        super(ClientHandshakeProcessorHybi00, self).__init__()
 
         self._socket = socket
         self._options = options
@@ -844,6 +843,13 @@ class EchoClient(object):
 
             request = ClientRequest(self._socket)
 
+            version_map = {
+                _PROTOCOL_VERSION_HYBI08: common.VERSION_HYBI08,
+                _PROTOCOL_VERSION_HYBI13: common.VERSION_HYBI13,
+                _PROTOCOL_VERSION_HYBI00: common.VERSION_HYBI00,
+                _PROTOCOL_VERSION_HIXIE75: common.VERSION_HIXIE75}
+            request.ws_version = version_map[version]
+
             if (version == _PROTOCOL_VERSION_HYBI08 or
                 version == _PROTOCOL_VERSION_HYBI13):
                 stream_option = StreamOptions()
