@@ -1,4 +1,4 @@
-# Copyright 2011, Google Inc.
+# Copyright 2012, Google Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -28,7 +28,11 @@
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
-"""Stream class for IETF HyBi latest WebSocket protocol.
+"""This file provides classes and helper functions for parsing/building frames
+of the WebSocket protocol (RFC 6455).
+
+Specification:
+http://tools.ietf.org/html/rfc6455
 """
 
 
@@ -204,7 +208,17 @@ class FragmentedFrameBuilder(object):
 def _create_control_frame(opcode, body, mask, frame_filters):
     frame = Frame(opcode=opcode, payload=body)
 
-    return _filter_and_format_frame_object(frame, mask, frame_filters)
+    for frame_filter in frame_filters:
+        frame_filter.filter(frame)
+
+    if len(frame.payload) > 125:
+        raise BadOperationException(
+            'Payload data size of control frames must be 125 bytes or less')
+
+    header = create_header(
+        frame.opcode, len(frame.payload), frame.fin,
+        frame.rsv1, frame.rsv2, frame.rsv3, mask)
+    return _build_frame(header, frame.payload, mask)
 
 
 def create_ping_frame(body, mask=False, frame_filters=[]):
@@ -238,7 +252,9 @@ class StreamOptions(object):
 
 
 class Stream(StreamBase):
-    """Stream of WebSocket messages."""
+    """A class for parsing/building frames of the WebSocket protocol
+    (RFC 6455).
+    """
 
     def __init__(self, request, options):
         """Constructs an instance.
@@ -322,9 +338,11 @@ class Stream(StreamBase):
                 length_encoding_bytes = 2
 
         if not valid_length_encoding:
-            logger.warning('Payload length is not encoded using the '
-                           'minimal number of bytes (%d is encoded using '
-                           '%d bytes)', payload_length, length_encoding_bytes)
+            self._logger.warning(
+                'Payload length is not encoded using the minimal number of '
+                'bytes (%d is encoded using %d bytes)',
+                payload_length,
+                length_encoding_bytes)
 
         if mask == 1:
             masking_nonce = self.receive_bytes(4)
@@ -351,8 +369,8 @@ class Stream(StreamBase):
 
         Raises:
             BadOperationException: when called on a server-terminated
-                connection or called with inconsistent message type or binary
-                parameter.
+                connection or called with inconsistent message type or
+                binary parameter.
         """
 
         if self._request.server_terminated:
@@ -400,6 +418,15 @@ class Stream(StreamBase):
 
             frame = self._receive_frame_as_frame_object()
 
+            # Check the constraint on the payload size for control frames
+            # before extension processes the frame.
+            # See also http://tools.ietf.org/html/rfc6455#section-5.5
+            if (common.is_control_opcode(frame.opcode) and
+                len(frame.payload) > 125):
+                raise InvalidFrameException(
+                    'Payload data size of control frames must be 125 bytes or '
+                    'less')
+
             for frame_filter in self._options.incoming_frame_filters:
                 frame_filter.filter(frame)
 
@@ -442,12 +469,6 @@ class Stream(StreamBase):
                 if frame.fin:
                     # Unfragmented frame
 
-                    if (common.is_control_opcode(frame.opcode) and
-                        len(frame.payload) > 125):
-                        raise InvalidFrameException(
-                            'Application data size of control frames must be '
-                            '125 bytes or less')
-
                     self._original_opcode = frame.opcode
                     message = frame.payload
                 else:
@@ -480,7 +501,11 @@ class Stream(StreamBase):
                 # - no application data: no code no reason
                 # - 2 octet of application data: has code but no reason
                 # - 3 or more octet of application data: both code and reason
-                if len(message) == 1:
+                if len(message) == 0:
+                    self._logger.debug('Received close frame (empty body)')
+                    self._request.ws_close_code = (
+                        common.STATUS_NO_STATUS_RECEIVED)
+                elif len(message) == 1:
                     raise InvalidFrameException(
                         'If a close frame has status code, the length of '
                         'status code must be 2 octet')
@@ -499,22 +524,28 @@ class Stream(StreamBase):
 
                 if self._request.server_terminated:
                     self._logger.debug(
-                        'Received ack for server-initiated closing '
-                        'handshake')
+                        'Received ack for server-initiated closing handshake')
                     return None
 
                 self._logger.debug(
                     'Received client-initiated closing handshake')
 
-                code = common.STATUS_NORMAL
+                code = common.STATUS_NORMAL_CLOSURE
                 reason = ''
                 if hasattr(self._request, '_dispatcher'):
                     dispatcher = self._request._dispatcher
                     code, reason = dispatcher.passive_closing_handshake(
                         self._request)
+                    if code is None and reason is not None and len(reason) > 0:
+                        self._logger.warning(
+                            'Handler specified reason despite code being None')
+                        reason = ''
+                    if reason is None:
+                        reason = ''
                 self._send_closing_handshake(code, reason)
                 self._logger.debug(
-                    'Sent ack for client-initiated closing handshake')
+                    'Sent ack for client-initiated closing handshake '
+                    '(code=%r, reason=%r)', code, reason)
                 return None
             elif self._original_opcode == common.OPCODE_PING:
                 try:
@@ -563,17 +594,21 @@ class Stream(StreamBase):
                     'Opcode %d is not supported' % self._original_opcode)
 
     def _send_closing_handshake(self, code, reason):
-        if code >= (1 << 16) or code < 0:
-            raise BadOperationException('Status code is out of range')
-
-        encoded_reason = reason.encode('utf-8')
-        if len(encoded_reason) + 2 > 125:
-            raise BadOperationException(
-                'Application data size of close frames must be 125 bytes or '
-                'less')
+        body = ''
+        if code is not None:
+            if (code > common.STATUS_USER_PRIVATE_MAX or
+                code < common.STATUS_NORMAL_CLOSURE):
+                raise BadOperationException('Status code is out of range')
+            if (code == common.STATUS_NO_STATUS_RECEIVED or
+                code == common.STATUS_ABNORMAL_CLOSURE or
+                code == common.STATUS_TLS_HANDSHAKE):
+                raise BadOperationException('Status code is reserved pseudo '
+                    'code')
+            encoded_reason = reason.encode('utf-8')
+            body = struct.pack('!H', code) + encoded_reason
 
         frame = create_close_frame(
-            struct.pack('!H', code) + encoded_reason,
+            body,
             self._options.mask_send,
             self._options.outgoing_frame_filters)
 
@@ -581,16 +616,37 @@ class Stream(StreamBase):
 
         self._write(frame)
 
-    def close_connection(self, code=common.STATUS_NORMAL, reason=''):
-        """Closes a WebSocket connection."""
+    def close_connection(self, code=common.STATUS_NORMAL_CLOSURE, reason=''):
+        """Closes a WebSocket connection.
+
+        Args:
+            code: Status code for close frame. If code is None, a close
+                frame with empty body will be sent.
+            reason: string representing close reason.
+        Raises:
+            BadOperationException: when reason is specified with code None
+            or reason is not an instance of both str and unicode.
+        """
 
         if self._request.server_terminated:
             self._logger.debug(
                 'Requested close_connection but server is already terminated')
             return
 
+        if code is None:
+            if reason is not None and len(reason) > 0:
+                raise BadOperationException(
+                    'close reason must not be specified if code is None')
+            reason = ''
+        else:
+            if not isinstance(reason, str) and not isinstance(reason, unicode):
+                raise BadOperationException(
+                    'close reason must be an instance of str or unicode')
+
         self._send_closing_handshake(code, reason)
-        self._logger.debug('Sent server-initiated closing handshake')
+        self._logger.debug(
+            'Sent server-initiated closing handshake (code=%r, reason=%r)',
+            code, reason)
 
         if (code == common.STATUS_GOING_AWAY or
             code == common.STATUS_PROTOCOL_ERROR):
@@ -613,10 +669,6 @@ class Stream(StreamBase):
         # note: mod_python Connection (mp_conn) doesn't have close method.
 
     def send_ping(self, body=''):
-        if len(body) > 125:
-            raise ValueError(
-                'Application data size of control frames must be 125 bytes or '
-                'less')
         frame = create_ping_frame(
             body,
             self._options.mask_send,
@@ -626,10 +678,6 @@ class Stream(StreamBase):
         self._ping_queue.append(body)
 
     def _send_pong(self, body):
-        if len(body) > 125:
-            raise ValueError(
-                'Application data size of control frames must be 125 bytes or '
-                'less')
         frame = create_pong_frame(
             body,
             self._options.mask_send,
