@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright 2011, Google Inc.
+# Copyright 2012, Google Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,7 @@ import set_sys_path  # Update sys.path to locate mod_pywebsocket module.
 
 from mod_pywebsocket import common
 from mod_pywebsocket.extensions import DeflateFrameExtensionProcessor
+from mod_pywebsocket.extensions import PerFrameCompressionExtensionProcessor
 from mod_pywebsocket import msgutil
 from mod_pywebsocket.stream import InvalidUTF8Exception
 from mod_pywebsocket.stream import Stream
@@ -68,8 +69,16 @@ def _mask_hybi(frame):
     return _MASKING_NONCE + result.tostring()
 
 
+def _install_extension_processor(processor, request, stream_options):
+    response = processor.get_extension_response()
+    if response is not None:
+        processor.setup_stream_options(stream_options)
+        request.ws_extension_processors.append(processor)
+
+
 def _create_request_from_rawdata(
-    read_data, deflate_stream=False, deflate_frame_request=None):
+    read_data, deflate_stream=False, deflate_frame_request=None,
+    perframe_compression_request=None):
     req = mock.MockRequest(connection=mock.MockConn(''.join(read_data)))
     req.ws_version = common.VERSION_HYBI_LATEST
     stream_options = StreamOptions()
@@ -77,10 +86,12 @@ def _create_request_from_rawdata(
     req.ws_extension_processors = []
     if deflate_frame_request is not None:
         processor = DeflateFrameExtensionProcessor(deflate_frame_request)
-        response = processor.get_extension_response()
-        if response is not None:
-            processor.setup_stream_options(stream_options)
-            req.ws_extension_processors.append(processor)
+        _install_extension_processor(processor, req, stream_options)
+    elif perframe_compression_request is not None:
+        processor = PerFrameCompressionExtensionProcessor(
+                        perframe_compression_request)
+        _install_extension_processor(processor, req, stream_options)
+
     req.ws_stream = Stream(req, stream_options)
     return req
 
@@ -277,20 +288,20 @@ class MessageTest(unittest.TestCase):
         """
 
         extension = common.ExtensionParameter(common.DEFLATE_FRAME_EXTENSION)
-        # window_bits less than 8 is illegal.
-        extension.add_parameter('window_bits', '7')
+        # max_window_bits less than 8 is illegal.
+        extension.add_parameter('max_window_bits', '7')
         processor = DeflateFrameExtensionProcessor(extension)
         self.assertEqual(None, processor.get_extension_response())
 
         extension = common.ExtensionParameter(common.DEFLATE_FRAME_EXTENSION)
-        # window_bits greater than 15 is illegal.
-        extension.add_parameter('window_bits', '16')
+        # max_window_bits greater than 15 is illegal.
+        extension.add_parameter('max_window_bits', '16')
         processor = DeflateFrameExtensionProcessor(extension)
         self.assertEqual(None, processor.get_extension_response())
 
         extension = common.ExtensionParameter(common.DEFLATE_FRAME_EXTENSION)
-        # Non integer window_bits is illegal.
-        extension.add_parameter('window_bits', 'foobar')
+        # Non integer max_window_bits is illegal.
+        extension.add_parameter('max_window_bits', 'foobar')
         processor = DeflateFrameExtensionProcessor(extension)
         self.assertEqual(None, processor.get_extension_response())
 
@@ -305,8 +316,8 @@ class MessageTest(unittest.TestCase):
         processor = DeflateFrameExtensionProcessor(extension)
         processor.set_response_window_bits(8)
         response = processor.get_extension_response()
-        self.assertTrue(response.has_parameter('window_bits'))
-        self.assertEqual('8', response.get_parameter_value('window_bits'))
+        self.assertTrue(response.has_parameter('max_window_bits'))
+        self.assertEqual('8', response.get_parameter_value('max_window_bits'))
 
         extension = common.ExtensionParameter(common.DEFLATE_FRAME_EXTENSION)
         processor = DeflateFrameExtensionProcessor(extension)
@@ -316,6 +327,33 @@ class MessageTest(unittest.TestCase):
         self.assertTrue(
             response.get_parameter_value('no_context_takeover') is None)
 
+    def test_send_message_perframe_compress_deflate(self):
+        compress = zlib.compressobj(
+            zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -zlib.MAX_WBITS)
+        extension = common.ExtensionParameter(
+            common.PERFRAME_COMPRESSION_EXTENSION)
+        extension.add_parameter('method', 'deflate')
+        request = _create_request_from_rawdata(
+                      '', perframe_compression_request=extension)
+        msgutil.send_message(request, 'Hello')
+        msgutil.send_message(request, 'World')
+
+        expected = ''
+
+        compressed_hello = compress.compress('Hello')
+        compressed_hello += compress.flush(zlib.Z_SYNC_FLUSH)
+        compressed_hello = compressed_hello[:-4]
+        expected += '\xc1%c' % len(compressed_hello)
+        expected += compressed_hello
+
+        compressed_world = compress.compress('World')
+        compressed_world += compress.flush(zlib.Z_SYNC_FLUSH)
+        compressed_world = compressed_world[:-4]
+        expected += '\xc1%c' % len(compressed_world)
+        expected += compressed_world
+
+        self.assertEqual(expected, request.connection.written_data())
+
     def test_receive_message(self):
         request = _create_request(
             ('\x81\x85', 'Hello'), ('\x81\x86', 'World!'))
@@ -341,6 +379,16 @@ class MessageTest(unittest.TestCase):
             ('\x81\xff\x00\x00\x00\x00\x00\x01\x00\x00', payload))
         self.assertEqual(payload, msgutil.receive_message(request))
 
+    def test_receive_length_not_encoded_using_minimal_number_of_bytes(self):
+        # Log warning on receiving bad payload length field that doesn't use
+        # minimal number of bytes but continue processing.
+
+        payload = 'a'
+        # 1 byte can be represented without extended payload length field.
+        request = _create_request(
+            ('\x81\xff\x00\x00\x00\x00\x00\x00\x00\x01', payload))
+        self.assertEqual(payload, msgutil.receive_message(request))
+
     def test_receive_message_unicode(self):
         request = _create_request(('\x81\x83', '\xe6\x9c\xac'))
         # U+672c is encoded as e6,9c,ac in UTF-8
@@ -481,6 +529,38 @@ class MessageTest(unittest.TestCase):
 
         self.assertEqual(None, msgutil.receive_message(request))
 
+    def test_receive_message_deflate_frame_client_using_smaller_window(self):
+        """Test that frames coming from a client which is using smaller window
+        size that the server are correctly received.
+        """
+
+        # Using the smallest window bits of 8 for generating input frames.
+        compress = zlib.compressobj(
+            zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -8)
+
+        data = ''
+
+        # Use a frame whose content is bigger than the clients' DEFLATE window
+        # size before compression. The content mainly consists of 'a' but
+        # repetition of 'b' is put at the head and tail so that if the window
+        # size is big, the head is back-referenced but if small, not.
+        payload = 'b' * 64 + 'a' * 1024 + 'b' * 64
+        compressed_hello = compress.compress(payload)
+        compressed_hello += compress.flush(zlib.Z_SYNC_FLUSH)
+        compressed_hello = compressed_hello[:-4]
+        data += '\xc1%c' % (len(compressed_hello) | 0x80)
+        data += _mask_hybi(compressed_hello)
+
+        # Close frame
+        data += '\x88\x8a' + _mask_hybi(struct.pack('!H', 1000) + 'Good bye')
+
+        extension = common.ExtensionParameter(common.DEFLATE_FRAME_EXTENSION)
+        request = _create_request_from_rawdata(
+            data, deflate_frame_request=extension)
+        self.assertEqual(payload, msgutil.receive_message(request))
+
+        self.assertEqual(None, msgutil.receive_message(request))
+
     def test_receive_message_deflate_frame_comp_bit(self):
         compress = zlib.compressobj(
             zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -zlib.MAX_WBITS)
@@ -510,12 +590,93 @@ class MessageTest(unittest.TestCase):
         for i in xrange(3):
             self.assertEqual('Hello', msgutil.receive_message(request))
 
+    def test_receive_message_perframe_compression_frame(self):
+        compress = zlib.compressobj(
+            zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -zlib.MAX_WBITS)
+
+        data = ''
+
+        compressed_hello = compress.compress('Hello')
+        compressed_hello += compress.flush(zlib.Z_SYNC_FLUSH)
+        compressed_hello = compressed_hello[:-4]
+        data += '\xc1%c' % (len(compressed_hello) | 0x80)
+        data += _mask_hybi(compressed_hello)
+
+        compressed_websocket = compress.compress('WebSocket')
+        compressed_websocket += compress.flush(zlib.Z_FINISH)
+        compressed_websocket += '\x00'
+        data += '\xc1%c' % (len(compressed_websocket) | 0x80)
+        data += _mask_hybi(compressed_websocket)
+
+        compress = zlib.compressobj(
+            zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -zlib.MAX_WBITS)
+
+        compressed_world = compress.compress('World')
+        compressed_world += compress.flush(zlib.Z_SYNC_FLUSH)
+        compressed_world = compressed_world[:-4]
+        data += '\xc1%c' % (len(compressed_world) | 0x80)
+        data += _mask_hybi(compressed_world)
+
+        # Close frame
+        data += '\x88\x8a' + _mask_hybi(struct.pack('!H', 1000) + 'Good bye')
+
+        extension = common.ExtensionParameter(
+            common.PERFRAME_COMPRESSION_EXTENSION)
+        extension.add_parameter('method', 'deflate')
+        request = _create_request_from_rawdata(
+            data, perframe_compression_request=extension)
+        self.assertEqual('Hello', msgutil.receive_message(request))
+        self.assertEqual('WebSocket', msgutil.receive_message(request))
+        self.assertEqual('World', msgutil.receive_message(request))
+
+        self.assertEqual(None, msgutil.receive_message(request))
+
+    def test_send_longest_close(self):
+        reason = 'a' * 123
+        request = _create_request(
+            ('\x88\xfd',
+             struct.pack('!H', common.STATUS_NORMAL_CLOSURE) + reason))
+        request.ws_stream.close_connection(common.STATUS_NORMAL_CLOSURE,
+                                           reason)
+        self.assertEqual(request.ws_close_code, common.STATUS_NORMAL_CLOSURE)
+        self.assertEqual(request.ws_close_reason, reason)
+
+    def test_send_close_too_long(self):
+        request = _create_request()
+        self.assertRaises(msgutil.BadOperationException,
+                          Stream.close_connection,
+                          request.ws_stream,
+                          common.STATUS_NORMAL_CLOSURE,
+                          'a' * 124)
+
+    def test_send_close_inconsistent_code_and_reason(self):
+        request = _create_request()
+        # reason parameter must not be specified when code is None.
+        self.assertRaises(msgutil.BadOperationException,
+                          Stream.close_connection,
+                          request.ws_stream,
+                          None,
+                          'a')
+
     def test_send_ping(self):
         request = _create_request()
         msgutil.send_ping(request, 'Hello World!')
         self.assertEqual('\x89\x0cHello World!',
                          request.connection.written_data())
 
+    def test_send_longest_ping(self):
+        request = _create_request()
+        msgutil.send_ping(request, 'a' * 125)
+        self.assertEqual('\x89\x7d' + 'a' * 125,
+                         request.connection.written_data())
+
+    def test_send_ping_too_long(self):
+        request = _create_request()
+        self.assertRaises(msgutil.BadOperationException,
+                          msgutil.send_ping,
+                          request,
+                          'a' * 126)
+
     def test_receive_ping(self):
         """Tests receiving a ping control frame."""
 
@@ -536,6 +697,19 @@ class MessageTest(unittest.TestCase):
         self.assertEqual('World', msgutil.receive_message(request))
         self.assertTrue(request.called)
 
+    def test_receive_longest_ping(self):
+        request = _create_request(
+            ('\x89\xfd', 'a' * 125), ('\x81\x85', 'World'))
+        self.assertEqual('World', msgutil.receive_message(request))
+        self.assertEqual('\x8a\x7d' + 'a' * 125,
+                         request.connection.written_data())
+
+    def test_receive_ping_too_long(self):
+        request = _create_request(('\x89\xfe\x00\x7e', 'a' * 126))
+        self.assertRaises(msgutil.InvalidFrameException,
+                          msgutil.receive_message,
+                          request)
+
     def test_receive_pong(self):
         """Tests receiving a pong control frame."""
 
